@@ -8,7 +8,7 @@
 
 #include <asm/types.h>          /* For __uXX types */
 
-#define IP_VS_VERSION_CODE            0x010003
+#define IP_VS_VERSION_CODE            0x010004
 #define NVERSION(version)                       \
 	(version >> 16) & 0xFF,                 \
 	(version >> 8) & 0xFF,                  \
@@ -25,18 +25,6 @@
  */
 #define IP_VS_DEST_F_AVAILABLE        0x0001    /* Available tag */
 
-/*
- * The default IP_VS_TEMPLATE_TIMEOUT is a little larger than average
- * connection time plus IPVS TCP FIN timeout(2*60*HZ). Because the
- * template won't be released until its controlled masq entries are
- * expired.
- * If IP_VS_TEMPLATE_TIMEOUT is too less, the template will soon expire
- * and will be put in expire again and again, which requires additional
- * overhead. If it is too large, the same will always visit the same
- * server, which will make dynamic load imbalance worse.
- */
-#define IP_VS_TEMPLATE_TIMEOUT  6*60*HZ
-
 /*
  *      IPVS sync daemon states
  */
@@ -412,8 +400,6 @@ struct ip_vs_stats
  */
 struct ip_vs_conn {
 	struct list_head        c_list;         /* hashed list heads */
-	atomic_t                refcnt;         /* reference count */
-	struct timer_list       timer;          /* Expiration timer */
 
 	/* Protocol, addresses and port numbers */
 	__u32                   caddr;          /* client address */
@@ -424,12 +410,16 @@ struct ip_vs_conn {
 	__u16                   dport;
 	__u16                   protocol;       /* Which protocol (TCP/UDP) */
 
+	/* counter and timer */
+	atomic_t		refcnt;		/* reference count */
+	struct timer_list	timer;		/* Expiration timer */
+	volatile unsigned long	timeout;	/* timeout */
+	struct ip_vs_timeout_table *timeout_table;
+
 	/* Flags and state transition */
 	spinlock_t              lock;           /* lock for state transition */
 	volatile __u16          flags;          /* status flags */
 	volatile __u16          state;          /* state info */
-	volatile unsigned long  timeout;        /* timeout */
-	struct ip_vs_timeout_table *timeout_table;
 
 	/* Control members */
 	struct ip_vs_conn       *control;       /* Master control connection */
@@ -577,7 +567,16 @@ extern unsigned int check_for_ip_vs_out(struct sk_buff **skb_p,
 #ifndef CONFIG_IP_VS_TAB_BITS
 #define CONFIG_IP_VS_TAB_BITS   12
 #endif
-#define IP_VS_CONN_TAB_BITS     CONFIG_IP_VS_TAB_BITS
+/* make sure that IP_VS_CONN_TAB_BITS is located in [8, 20] */
+#if CONFIG_IP_VS_TAB_BITS < 8
+#define IP_VS_CONN_TAB_BITS	8
+#endif
+#if CONFIG_IP_VS_TAB_BITS > 20
+#define IP_VS_CONN_TAB_BITS	20
+#endif
+#if 8 <= CONFIG_IP_VS_TAB_BITS && CONFIG_IP_VS_TAB_BITS <= 20
+#define IP_VS_CONN_TAB_BITS	CONFIG_IP_VS_TAB_BITS
+#endif
 #define IP_VS_CONN_TAB_SIZE     (1 << IP_VS_CONN_TAB_BITS)
 #define IP_VS_CONN_TAB_MASK     (IP_VS_CONN_TAB_SIZE - 1)
 
@@ -726,7 +725,7 @@ extern void ip_vs_control_cleanup(void);
  *      IPVS sync daemon data and function prototypes
  *      (from ip_vs_sync.c)
  */
-extern int ip_vs_sync_state;
+extern volatile int ip_vs_sync_state;
 extern char ip_vs_mcast_ifn[IP_VS_IFNAME_MAXLEN];
 extern int start_sync_thread(int state, char *mcast_ifn);
 extern int stop_sync_thread(void);
@@ -736,9 +735,9 @@ extern void ip_vs_sync_conn(struct ip_vs_conn *cp);
 /*
  *      IPVS rate estimator prototypes (from ip_vs_est.c)
  */
-int ip_vs_new_estimator(struct ip_vs_stats *stats);
-void ip_vs_kill_estimator(struct ip_vs_stats *stats);
-void ip_vs_zero_estimator(struct ip_vs_stats *stats);
+extern int ip_vs_new_estimator(struct ip_vs_stats *stats);
+extern void ip_vs_kill_estimator(struct ip_vs_stats *stats);
+extern void ip_vs_zero_estimator(struct ip_vs_stats *stats);
 
 
 /*
