@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_conn.c,v 1.26 2002/05/30 13:00:20 wensong Exp $
+ * Version:     $Id: ip_vs_conn.c,v 1.27 2002/07/11 14:26:41 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -1348,7 +1348,7 @@ ip_vs_conn_getinfo(char *buffer, char **start, off_t offset, int length)
 				ntohl(cp->vaddr), ntohs(cp->vport),
 				ntohl(cp->daddr), ntohs(cp->dport),
 				ip_vs_state_name(cp->state),
-				cp->timer.expires-jiffies);
+				(cp->timer.expires-jiffies)/HZ);
 			len += sprintf(buffer+len, "%-127s\n", temp);
 			if (pos >= offset+length) {
 				ct_read_unlock_bh(idx);
