@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_sync.c,v 1.5 2001/12/11 12:41:29 wensong Exp $
+ * Version:     $Id: ip_vs_sync.c,v 1.6 2002/07/11 14:26:41 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *
@@ -171,6 +171,25 @@ static inline void ip_vs_sync_buff_release(struct ip_vs_sync_buff *sb)
 	kfree(sb);
 }
 
+/*
+ *	Get the current sync buffer if it has been created for more
+ *	than the specified time.
+ */
+static inline struct ip_vs_sync_buff *
+get_curr_sync_buff(unsigned long time)
+{
+	struct ip_vs_sync_buff *sb;
+
+	spin_lock(&curr_sb_lock);
+	if (curr_sb && (jiffies - curr_sb->firstuse > time)) {
+		sb = curr_sb;
+		curr_sb = NULL;
+	} else
+		sb = NULL;
+	spin_unlock(&curr_sb_lock);
+	return sb;
+}
+
 
 /*
  *      Add an ip_vs_conn information into the current sync_buff.
@@ -185,8 +204,8 @@ void ip_vs_sync_conn(struct ip_vs_conn *cp)
 	spin_lock(&curr_sb_lock);
 	if (!curr_sb) {
 		if (!(curr_sb=ip_vs_sync_buff_create())) {
-			IP_VS_ERR("ip_vs_sync_buff_create failed.\n");
 			spin_unlock(&curr_sb_lock);
+			IP_VS_ERR("ip_vs_sync_buff_create failed.\n");
 			return;
 		}
 	}
@@ -263,11 +282,15 @@ static void ip_vs_process_message(const char *buffer, const size_t buflen)
 				IP_VS_ERR("ip_vs_conn_new failed\n");
 				return;
 			}
-		} else
+			cp->state = ntohs(s->state);
+		} else if (!cp->dest) {
+			/* it is an entry created by the synchronization */
+			cp->state = ntohs(s->state);
 			cp->flags = ntohs(s->flags) | IP_VS_CONN_F_HASHED;
+		}	/* Note that we don't touch its state and flags
+			   if it is a normal entry. */
 
-		cp->state = ntohs(s->state);
-		if (cp->flags & IP_VS_CONN_F_SEQ_MASK) {
+		if (ntohs(s->flags) & IP_VS_CONN_F_SEQ_MASK) {
 			opt = (struct ip_vs_sync_conn_options *)&s[1];
 			memcpy(&cp->in_seq, opt, sizeof(*opt));
 			p += FULL_CONN_SIZE;
@@ -287,7 +310,7 @@ static void ip_vs_process_message(const char *buffer, const size_t buflen)
 
 
 /* ipvs sync daemon state */
-int ip_vs_sync_state = IP_VS_STATE_NONE;
+volatile int ip_vs_sync_state = IP_VS_STATE_NONE;
 
 /* multicast interface name */
 char ip_vs_mcast_ifn[IP_VS_IFNAME_MAXLEN];
@@ -564,22 +587,20 @@ static void sync_master_loop(void)
 		}
 
 		/* check if entries stay in curr_sb for a while */
-		spin_lock(&curr_sb_lock);
-		if (curr_sb && (jiffies - curr_sb->firstuse > 2*HZ)) {
-			m = curr_sb->mesg;
+		if ((sb = get_curr_sync_buff(2*HZ))) {
+			m = sb->mesg;
 			if (ip_vs_send_async(sock, (char *)m,
 					     m->size) != m->size)
 				IP_VS_ERR("ip_vs_send_async error\n");
-			ip_vs_sync_buff_release(curr_sb);
-			curr_sb = NULL;
+			ip_vs_sync_buff_release(sb);
 		}
-		spin_unlock(&curr_sb_lock);
 
 		if (stop_sync)
 			break;
 
 		__set_current_state(TASK_INTERRUPTIBLE);
 		schedule_timeout(HZ);
+		__set_current_state(TASK_RUNNING);
 	}
 
 	/* cleanup the sync_buff queue and the current sync_buff */
@@ -630,6 +651,7 @@ static void sync_backup_loop(void)
 
 		__set_current_state(TASK_INTERRUPTIBLE);
 		schedule_timeout(HZ);
+		__set_current_state(TASK_RUNNING);
 	}
 
 	/* release the sending multicast socket */
