@@ -2,7 +2,7 @@
  *      ipvsadm - IP Virtual Server ADMinistration program
  *                for IPVS NetFilter Module in kernel 2.4
  *
- *      Version: $Id: ipvsadm.c,v 1.17 2001/11/04 09:18:01 wensong Exp $
+ *      Version: $Id: ipvsadm.c,v 1.19 2002/07/11 14:26:44 wensong Exp $
  *
  *      Authors: Wensong Zhang <wensong@linuxvirtualserver.org>
  *               Peter Kese <peter.kese@ijs.si>
@@ -108,12 +108,6 @@
 #include <netinet/udp.h>
 #include <netinet/tcp.h>
 
-#ifdef HAVE_NET_IP_VS_H
-#include <net/ip_vs.h>
-#else
-#include "ip_vs.h"
-#endif
-
 #ifdef HAVE_POPT
 #include "popt.h"
 #define IPVS_OPTION_PROCESSING          "popt"
@@ -124,13 +118,13 @@
 #include "config_stream.h"
 #include "libipvs/libipvs.h"
 
-#define IPVSADM_VERSION_NO              "v" VERSION
-#define IPVSADM_VERSION_DATE            "2001/11/04"
-#define IPVSADM_VERSION         IPVSADM_VERSION_NO " " IPVSADM_VERSION_DATE
+#define IPVSADM_VERSION_NO		"v" VERSION
+#define IPVSADM_VERSION_DATE		"2002/07/09"
+#define IPVSADM_VERSION			IPVSADM_VERSION_NO " " IPVSADM_VERSION_DATE
 
-#define MINIMUM_IPVS_VERSION_MAJOR      0
-#define MINIMUM_IPVS_VERSION_MINOR      9
-#define MINIMUM_IPVS_VERSION_PATCH      6
+#define MINIMUM_IPVS_VERSION_MAJOR      1
+#define MINIMUM_IPVS_VERSION_MINOR      0
+#define MINIMUM_IPVS_VERSION_PATCH      4
 
 #ifndef IPVS_VERSION
 #define IPVS_VERSION(x,y,z) (((x)<<16)+((y)<<8)+(z))
@@ -240,43 +234,43 @@ static const char commands_v_options[NUMBER_OF_CMD][NUMBER_OF_OPT] =
 #define CONN_PROC_FILE  "/proc/net/ip_vs_conn"
 
 /* various parsing helpers & parsing functions */
-int str_is_digit(const char *str);
-int string_to_number(const char *s, int min, int max);
-int host_to_addr(const char *name, struct in_addr *addr);
-char * addr_to_host(struct in_addr *addr);
-char * addr_to_anyname(struct in_addr *addr);
-int service_to_port(const char *name, unsigned short proto);
-char * port_to_service(int port, unsigned short proto);
-char * port_to_anyname(int port, unsigned short proto);
-char * addrport_to_anyname(struct in_addr *addr, int port,
-			   unsigned short proto, unsigned int format);
-int parse_service(char *buf, u_int16_t proto,
+static int str_is_digit(const char *str);
+static int string_to_number(const char *s, int min, int max);
+static int host_to_addr(const char *name, struct in_addr *addr);
+static char * addr_to_host(struct in_addr *addr);
+static char * addr_to_anyname(struct in_addr *addr);
+static int service_to_port(const char *name, unsigned short proto);
+static char * port_to_service(unsigned short port, unsigned short proto);
+static char * port_to_anyname(unsigned short port, unsigned short proto);
+static char * addrport_to_anyname(struct in_addr *addr, unsigned short port,
+				  unsigned short proto, unsigned int format);
+static int parse_service(char *buf, u_int16_t proto,
 		  u_int32_t *addr, u_int16_t *port);
-int parse_netmask(char *buf, u_int32_t *addr);
-int parse_timeout(char *buf, int min, int max);
-unsigned int parse_fwmark(char *buf);
+static int parse_netmask(char *buf, u_int32_t *addr);
+static int parse_timeout(char *buf, int min, int max);
+static unsigned int parse_fwmark(char *buf);
 
 /* check the options based on the commands_v_options table */
 static void generic_opt_check(int command, int options);
 static void set_command(int *cmd, const int newcmd);
 static void set_option(unsigned int *options, unsigned int option);
 
-void tryhelp_exit(const char *program, const int exit_status);
-void usage_exit(const char *program, const int exit_status);
-void version_exit(int exit_status);
-void version(FILE *stream);
-void fail(int err, char *msg, ...);
+static void tryhelp_exit(const char *program, const int exit_status);
+static void usage_exit(const char *program, const int exit_status);
+static void version_exit(int exit_status);
+static void version(FILE *stream);
+static void fail(int err, char *msg, ...);
 
 /* various listing functions */
-void list_conn(unsigned int format);
-void list_service(struct ip_vs_rule_user *ur, unsigned int format);
-void list_all(unsigned int format);
-void list_timeout(void);
-void list_daemon(void);
+static void list_conn(unsigned int format);
+static void list_service(struct ip_vs_rule_user *ur, unsigned int format);
+static void list_all(unsigned int format);
+static void list_timeout(void);
+static void list_daemon(void);
 
-int modprobe_ipvs(void);
-void check_ipvs_version(void);
-int process_options(int argc, char **argv, int reading_stdin);
+static int modprobe_ipvs(void);
+static void check_ipvs_version(void);
+static int process_options(int argc, char **argv, int reading_stdin);
 
 
 int main(int argc, char **argv)
@@ -312,8 +306,9 @@ int main(int argc, char **argv)
 
 #ifdef HAVE_POPT
 
-int parse_options(int argc, char **argv, struct ip_vs_rule_user *ur,
-		  int *cmd, unsigned int *options, unsigned int *format)
+static int
+parse_options(int argc, char **argv, struct ip_vs_rule_user *ur,
+	      int *cmd, unsigned int *options, unsigned int *format)
 {
 	int c, parse;
 	poptContext context;
@@ -559,8 +554,9 @@ int parse_options(int argc, char **argv, struct ip_vs_rule_user *ur,
 
 #else /* HAVE_POPT */
 
-int parse_options(int argc, char **argv, struct ip_vs_rule_user *ur,
-		  int *cmd, unsigned int *options, unsigned int *format)
+static int
+parse_options(int argc, char **argv, struct ip_vs_rule_user *ur,
+	      int *cmd, unsigned int *options, unsigned int *format)
 {
 	int c, parse;
 	const char *optstring = "AEDCZSRaedlLhvt:u:f:s:M:p::w:r:gmicn";
@@ -800,7 +796,7 @@ int parse_options(int argc, char **argv, struct ip_vs_rule_user *ur,
 #endif /* HAVE_POPT */
 
 
-int restore_table(int argc, char **argv, int reading_stdin)
+static int restore_table(int argc, char **argv, int reading_stdin)
 {
 	int result=0;
 	dynamic_array_t *a;
@@ -819,7 +815,7 @@ int restore_table(int argc, char **argv, int reading_stdin)
 	return result;
 }
 
-int process_options(int argc, char **argv, int reading_stdin)
+static int process_options(int argc, char **argv, int reading_stdin)
 {
 	struct ip_vs_rule_user urule;
 	int cmd=IP_VS_SO_SET_NONE;
@@ -906,7 +902,7 @@ int process_options(int argc, char **argv, int reading_stdin)
 }
 
 
-int string_to_number(const char *s, int min, int max)
+static int string_to_number(const char *s, int min, int max)
 {
 	long number;
 	char *end;
@@ -925,25 +921,25 @@ int string_to_number(const char *s, int min, int max)
 /*
  * Parse the timeout value.
  */
-int parse_timeout(char *buf, int min, int max)
+static int parse_timeout(char *buf, int min, int max)
 {
 	int i;
 
 	/* it is just for parsing timeout of persistent service */
 	if (buf == NULL)
-		return IP_VS_TEMPLATE_TIMEOUT;
+		return IPVS_SVC_PERSISTENT_TIMEOUT;
 
 	if ((i=string_to_number(buf, min, max)) == -1)
 		fail(2, "invalid timeout value `%s' specified", buf);
 
-	return i * HZ;
+	return i;
 }
 
 
 /*
  * Parse IP fwmark from the argument.
  */
-unsigned int parse_fwmark(char *buf)
+static unsigned int parse_fwmark(char *buf)
 {
 	unsigned long l;
 	char *end;
@@ -963,7 +959,7 @@ unsigned int parse_fwmark(char *buf)
  * Return 0 if failed,
  *	  1 if addr read
  */
-int parse_netmask(char *buf, u_int32_t *addr)
+static int parse_netmask(char *buf, u_int32_t *addr)
 {
 	struct in_addr inaddr;
 
@@ -988,7 +984,8 @@ int parse_netmask(char *buf, u_int32_t *addr)
  * SERVICE_ADDR:   addr set
  * SERVICE_PORT:   port set
  */
-int parse_service(char *buf, u_int16_t proto, u_int32_t *addr, u_int16_t *port)
+static int
+parse_service(char *buf, u_int16_t proto, u_int32_t *addr, u_int16_t *port)
 {
 	char *portp;
 	long portn;
@@ -1086,7 +1083,7 @@ set_option(unsigned int *options, unsigned int option)
 }
 
 
-void tryhelp_exit(const char *program, const int exit_status)
+static void tryhelp_exit(const char *program, const int exit_status)
 {
 	fprintf(stderr, "Try `%s -h' or '%s --help' for more information.\n",
 		program, program);
@@ -1094,7 +1091,7 @@ void tryhelp_exit(const char *program, const int exit_status)
 }
 
 
-void usage_exit(const char *program, const int exit_status)
+static void usage_exit(const char *program, const int exit_status)
 {
 	FILE *stream;
 
@@ -1170,7 +1167,7 @@ void usage_exit(const char *program, const int exit_status)
 }
 
 
-void version_exit(const int exit_status)
+static void version_exit(const int exit_status)
 {
 	FILE *stream;
 
@@ -1185,7 +1182,7 @@ void version_exit(const int exit_status)
 }
 
 
-void version(FILE *stream)
+static void version(FILE *stream)
 {
 	fprintf(stream,
 		"ipvsadm " IPVSADM_VERSION " (compiled with "
@@ -1194,7 +1191,7 @@ void version(FILE *stream)
 }
 
 
-void fail(int err, char *msg, ...)
+static void fail(int err, char *msg, ...)
 {
 	va_list args;
 
@@ -1206,7 +1203,7 @@ void fail(int err, char *msg, ...)
 }
 
 
-int modprobe_ipvs(void)
+static int modprobe_ipvs(void)
 {
 	char *argv[] = { "/sbin/modprobe", "-s", "-k", "--", "ip_vs", NULL };
 	int child;
@@ -1228,7 +1225,7 @@ int modprobe_ipvs(void)
 }
 
 
-void check_ipvs_version(void)
+static void check_ipvs_version(void)
 {
 	/* verify the IPVS version */
 	if (ipvs_info.version <
@@ -1248,7 +1245,7 @@ void check_ipvs_version(void)
 }
 
 
-void print_conn(char *buf, unsigned int format)
+static void print_conn(char *buf, unsigned int format)
 {
 	char            protocol[8];
 	unsigned short  proto;
@@ -1264,9 +1261,10 @@ void print_conn(char *buf, unsigned int format)
 	int n;
 	unsigned long temp1, temp2, temp3;
 	char *cname, *vname, *dname;
-	unsigned long minutes, seconds, sec100s;
+	unsigned int	minutes, seconds;
+	char		expire_str[12];
 
-	if ((n = sscanf(buf, " %s %lX %hX %lX %hX %lX %hX %s %d",
+	if ((n = sscanf(buf, "%s %lX %hX %lX %hX %lX %hX %s %d",
 			protocol, &temp1, &cport, &temp2, &vport,
 			&temp3, &dport, state, &expires)) == -1)
 		exit(1);
@@ -1289,13 +1287,12 @@ void print_conn(char *buf, unsigned int format)
 	if (!(dname=addrport_to_anyname(&daddr, dport, proto, format)))
 		exit(1);
 
-	sec100s = ((expires % HZ) * 100) / HZ;
-	seconds = (expires / HZ) % 60;
-	minutes = expires / (60 * HZ);
+	seconds = expires % 60;
+	minutes = expires / 60;
+	sprintf(expire_str, "%02d:%02d", minutes, seconds);
 
-	printf("%-3s %02ld:%02ld.%02ld %-11s %-17s %-17s %-17s\n",
-	       protocol, minutes, seconds, sec100s, state,
-	       cname, vname, dname);
+	printf("%-3s %-6s %-11s %-18s %-18s %s\n",
+	       protocol, expire_str, state, cname, vname, dname);
 
 	free(cname);
 	free(vname);
@@ -1321,7 +1318,7 @@ void list_conn(unsigned int format)
 		exit(1);
 	}
 	printf("IPVS connection entries\n");
-	printf("pro expire   %-11s %-17s %-17s %-17s\n",
+	printf("pro expire %-11s %-18s %-18s %s\n",
 	       "state", "source", "virtual", "destination");
 
 	/*
@@ -1447,7 +1444,7 @@ print_service(struct ip_vs_service_user *svc, unsigned int format)
 	if (format & FMT_RULE) {
 		printf("-A %s -s %s", svc_name, svc->sched_name);
 		if (svc->flags & IP_VS_SVC_F_PERSISTENT) {
-			printf(" -p %u", svc->timeout/HZ);
+			printf(" -p %u", svc->timeout);
 			if (svc->netmask != (unsigned long int) 0xffffffff) {
 				struct in_addr mask;
 				mask.s_addr = svc->netmask;
@@ -1471,7 +1468,7 @@ print_service(struct ip_vs_service_user *svc, unsigned int format)
 	} else {
 		printf("%s %s", svc_name, svc->sched_name);
 		if (svc->flags & IP_VS_SVC_F_PERSISTENT) {
-			printf(" persistent %u", svc->timeout/HZ);
+			printf(" persistent %u", svc->timeout);
 			if (svc->netmask != (unsigned long int) 0xffffffff) {
 				struct in_addr mask;
 				mask.s_addr = svc->netmask;
@@ -1523,7 +1520,7 @@ print_service(struct ip_vs_service_user *svc, unsigned int format)
 }
 
 
-void list_service(struct ip_vs_rule_user *ur, unsigned int format)
+static void list_service(struct ip_vs_rule_user *ur, unsigned int format)
 {
 	struct ip_vs_service_user *svc;
 
@@ -1539,7 +1536,7 @@ void list_service(struct ip_vs_rule_user *ur, unsigned int format)
 }
 
 
-void list_all(unsigned int format)
+static void list_all(unsigned int format)
 {
 	struct ip_vs_get_services *get;
 	int i;
@@ -1567,12 +1564,12 @@ void list_timeout(void)
 	if (!(u = ipvs_get_timeouts()))
 		exit(1);
 	printf("Timeout (tcp tcpfin udp): %d %d %d\n",
-	       u->tcp_timeout/HZ, u->tcp_fin_timeout/HZ, u->udp_timeout/HZ);
+	       u->tcp_timeout, u->tcp_fin_timeout, u->udp_timeout);
 	free(u);
 }
 
 
-void list_daemon(void)
+static void list_daemon(void)
 {
 	struct ip_vs_daemon_user *u;
 
@@ -1612,7 +1609,7 @@ int host_to_addr(const char *name, struct in_addr *addr)
 }
 
 
-char * addr_to_host(struct in_addr *addr)
+static char * addr_to_host(struct in_addr *addr)
 {
 	struct hostent *host;
 
@@ -1624,7 +1621,7 @@ char * addr_to_host(struct in_addr *addr)
 }
 
 
-char * addr_to_anyname(struct in_addr *addr)
+static char * addr_to_anyname(struct in_addr *addr)
 {
 	char *name;
 
@@ -1650,7 +1647,7 @@ int service_to_port(const char *name, unsigned short proto)
 }
 
 
-char * port_to_service(int port, unsigned short proto)
+static char * port_to_service(unsigned short port, unsigned short proto)
 {
 	struct servent *service;
 
@@ -1665,7 +1662,7 @@ char * port_to_service(int port, unsigned short proto)
 }
 
 
-char * port_to_anyname(int port, unsigned short proto)
+static char * port_to_anyname(unsigned short port, unsigned short proto)
 {
 	char *name;
 	static char buf[10];
@@ -1673,14 +1670,15 @@ char * port_to_anyname(int port, unsigned short proto)
 	if ((name = port_to_service(port, proto)) != NULL)
 		return name;
 	else {
-		sprintf(buf, "%d", port);
+		sprintf(buf, "%u", port);
 		return buf;
 	}
 }
 
 
-char * addrport_to_anyname(struct in_addr *addr, int port,
-			   unsigned short proto, unsigned int format)
+static char *
+addrport_to_anyname(struct in_addr *addr, unsigned short port,
+		    unsigned short proto, unsigned int format)
 {
 	char *buf;
 
@@ -1699,7 +1697,7 @@ char * addrport_to_anyname(struct in_addr *addr, int port,
 }
 
 
-int str_is_digit(const char *str)
+static int str_is_digit(const char *str)
 {
 	size_t offset;
 	size_t top;
